#!/usr/bin/env python3
# vim: fdm=indent ts=4

import h5py
from itertools import combinations, zip_longest
import matplotlib.pyplot as plt
import numpy as np

import aa_generate_beacon as beacon
import lib

if __name__ == "__main__":
    from os import path
    import sys

    fname = "ZH_airshower/mysim.sry"

    show_plots = True
    ref_ant_id = None # leave None for all baselines

    ####
    fname_dir = path.dirname(fname)
    antennas_fname = path.join(fname_dir, beacon.antennas_fname)

    # Read in antennas from file
    f_beacon, tx, antennas = beacon.read_beacon_hdf5(antennas_fname)

    # run over all baselines
    if ref_ant_id is None:
        print("Doing all baselines")
        baselines = list(combinations(antennas,2))
    # use ref_ant
    else:
        ref_ant = antennas[ref_ant_idx]
        baselines = list(zip_longest([], antennas, fillvalue=ref_ant))

    freq_names = antennas[0].beacon_info.keys()
    if len(freq_names) > 1:
        raise NotImplementedError

    freq_name = next(iter(freq_names))

    # Determine integer multiple of periods to shift
    # and True phase differences
    time_diffs = np.empty( (len(baselines), 3) )
    for i, base in enumerate(baselines):
        # which traces to keep track of
        traces = [ base[0].E_AxB, base[1].E_AxB ]

        sampling_dt = (base[1].t[1] - base[1].t[0]) # ns
        # how many samples do we need to shift
        ks, maxima = lib.coherence_sum_maxima(traces[0], traces[1])
        max_idx = np.argmax(maxima)
        best_k = ks[max_idx]
        delta_t_coherence = sampling_dt*best_k # ns

        print('A1:', base[0].name, 'A2:', base[1].name, "K:", best_k, '= [ns]', delta_t_coherence)

        # get the amount of periods to move
        f_beacon = base[0].beacon_info[freq_name]['freq']
        k_period, rest = np.divmod(delta_t_coherence, 1/f_beacon)

        # always keep the reference before traces[1]
        if rest < 0:
            k_period -= 1

        # Get true phase diffs
        try:
            true_phases = np.array([ant.beacon_info[freq_name]['true_phase'] for ant in base])
            true_phases_diff = lib.phase_mod(true_phases[0] - true_phases[1])
        except IndexError:
            # freq_name not in beacon_info
            # or true_phase not determined yet
            true_phases_diff = np.nan

        # save k_period with antenna names
        time_diffs[i] = [true_phases_diff, k_period, f_beacon]

        # Plotting for one or two iterations
        if show_plots and i in [ 0, 1 ]:
            print('i',i,'k[T]',k_period, 'rest[ns]',rest, 'T[ns]',1/f_beacon)

            # Show correlation maxima plot
            if not True:
                fig, ax = plt.subplots()
                ax.set_title(f"Correlation Maxima {i}")
                ax.set_xlabel("k")
                ax.set_ylabel("Maximum correlation")
                ax.plot(ks, maxima)
                ax.plot(best_k, maxima[max_idx], marker='X')

            # Delta between first timestamp from both antennas
            delta_t_antennas = base[0].t[0] - base[1].t[0]

            # Delta t due to the beacon
            if true_phases_diff is np.nan:
                true_phases_diff = 0
            delta_t_beacon = true_phases_diff/(2*np.pi*f_beacon)

            print("t0[ns]", delta_t_antennas, "t_beacon[ns]", delta_t_beacon, "phase", true_phases_diff)
            fig, ax = plt.subplots()
            ax.set_xlabel('t')
            ax.plot(base[0].t, traces[0], label=f'Reference {base[0].name}', alpha=0.5)
            # plot vertical lines indicating f_beacon
            min_t, max_t = base[0].t[0], base[0].t[-1]
            N_lines = int( (max_t - min_t)*f_beacon) +1
            for i, t in enumerate(np.arange(N_lines)/f_beacon):
                ax.axvline( min_t + t, color='k', alpha=0.5, label=None if i!=0 else 'P_beacon')

            ax.plot(base[1].t + delta_t_antennas, traces[1], label=f'Original {base[1].name} (t0 removed)', alpha=0.4, marker='+', ms=5)
            ax.plot(base[1].t + delta_t_antennas + k_period/f_beacon + rest, traces[1], label='Coherence', alpha=0.3, marker='x', ms=5)
            ax.plot(base[1].t + delta_t_antennas + k_period/f_beacon + delta_t_beacon, traces[1], label='Beacon only + Periods', alpha=0.6)

            ax.legend()

    # Save integer periods to antennas
    beacon.write_baseline_time_diffs_hdf5(antennas_fname, baselines, time_diffs[:,0], time_diffs[:,1], time_diffs[:,2])

    # Report back to CLI
    print("Period Multiples resolved and written to ", antennas_fname)
    plt.show()
